"CRDS / FAGE intercomparison workup script"


"File paths"

CRDS   = 'C:\Users\cm11ab\Desktop\Working_File\CRDS\CRDS_Data'
FAGE   = 'C:\Users\cm11ab\Desktop\Working_File\CRDS\FAGE_Data'
Output = 'C:\Users\cm11ab\Desktop\Working_File\CRDS'


"Switches and variables"

### Saving ###
Save       = True   ### If true, will save plots and excel files
Comparison = False   ### If true, will read and overlay FAGE data, else will just analyse cavity
Filtering  = True   ### If true, will filter CRDS data by checking each point lies within a range (set below) of the points either side of it
limit      = 0.99   ### Agreement factor required for point to be accepted

### RDT filter ###
RDT_upper = 240    ### Upper RDT limit (us)
RDT_lower = 150    ### Lower RDT limit (us)

### Time range - begins at 0 ###
time_start = 000     ### Set the time in seconds to begin analysis
time_end   = 4000     ### Set the time in seconds to end analysis

### Baseline correction ###
baseline_gradient   = -5.90E-10  ### Set the gradient of the line to be used as the baseline

baseline_intercept  = 2.22961e-4       ### Set the intercept of the line to be used as the baseline

### Averaging ###
averaging_interval  = 5       ### Set the number of seconds for which to average the data
minimum_points      = 3       ### Set the minimum number of points to be averaged together - will prevent single data points being used as an average with zero error  

### FAGE time offset ###
time_offset = 739             ### Define the time, in seconds, to be added or removed from the FAGE timestamps to align the two data sets 


"Define constants"

speed_of_light = 29979245800  ### Speed of light (cm s-1)
sigma          = 1.46E-20     ### CH3O2                ###1.23443E-19, 3.91E-20, 3.456E-20  ### Absorption cross section (cm2) 


"Convert file paths to python readable strings"

FAGE = FAGE.replace('\\', '/') 
FAGE = FAGE + '/'

CRDS = CRDS.replace('\\', '/') 
CRDS = CRDS + '/'

Output = Output.replace('\\', '/')
Output = Output + '/'


"Import modules"

import os
import numpy as np
from nptdms import TdmsFile as TF
import pandas as pd
from matplotlib import pyplot as plt


"Define functions"

def convert_timestamp(time):
    hour   = int(time[:2]) * 60**2
    minute = int(time[3:5]) * 60
    second = int(time[6:8])
    ms     = float(time[-2:]) / 100
    timeseconds = float(hour + minute + second + ms)
    return timeseconds
    
def intervallic_average(time, signal, absorption, interval):      ### Averages the data given to it in blocks of length 'interval' in seconds
    block           = []                              ### Initialises lists
    time_index      = []
    time_list       = []
    interval_time   = []
    signal_average  = []
    signal_dev      = []
    absorption_average = []
    absorption_dev = []
    points_averaged = []
    iterations      = 1
    
    for i, v in enumerate(time):                     ### Iterates through the time series data
        
        if len(block) < 1:                           ### Prevents list indexing errors - will never want blocks less than 2 events in length anyway... right?
            
            block.append(v)                          ### Starts building the block of times
            time_index.append(i)                     ### Starts building the index list of the times to be averaged                  
        
        elif block[-1] < (time_start + (iterations * interval)):       ### If the gap between the start and end of the block is lower than the time interval, keep adding times to the block
            
            block.append(v)
            time_index.append(i)
        
        elif block[-1] >= (time_start + (iterations * interval)):        ### Once exceeded, process the block
            
            time_list.append(np.median(block))       ### Finds the median time of the block - this will be the time value used for the data point
            avgStart = time_index[0]                 ### Determines the index of the start   
            avgEnd = time_index[-1]                  ### Determines the index of the end
            signal_average.append(np.average(signal[avgStart:avgEnd+1]))   ### Uses the start and end index values to extract the relevant block from the data list
            signal_dev.append(np.std(signal[avgStart:avgEnd+1]))           ### Calculates standard deviation   
            absorption_average.append(np.average(absorption[avgStart:avgEnd+1]))
            absorption_dev.append(np.std(absorption[avgStart:avgEnd+1]))
            interval_time.append((len(signal_average) * interval))
            points_averaged.append(len(block))       ### Counts the number of points averaged in the block
            time_index = []                          ### Resets the block back to an empty list for the next iteration 
            block = []                               ###
            iterations += 1
            
    
    return time_list, signal_average, signal_dev, points_averaged, absorption_average, absorption_dev     ### Outputs a list of the median times and a list of averaged data

def Absorbance(t, t0, sigma, SoL):                        ### Calculates the concentration using the absorption cross section (sigma) and the ringdown time of the signal (t) and baseline (t0)
    alpha = (1/float(SoL)) * ((1/t) - (1/t0))
    C = alpha / sigma
    return alpha, C


"Begin analysis - extract CRDS data"

for file in os.listdir(CRDS):

    CRDS_file = TF(CRDS + file)

date_timestamp_array = CRDS_file.object('Averages', 'Date and time stamp').data
date = str(date_timestamp_array[0])[:10]            ### Reads the first line of the timestamp array to extract a date    
time = str(date_timestamp_array[0])[-11:-3]        ### Reads the time from the line
timestamp_array = np.array([convert_timestamp(t[-11:]) for t in date_timestamp_array])    ### Converts the unicode date + timestamp strings into seconds after midnight
timestamp_to_zero = timestamp_array - timestamp_array[0]    
#event_array = a.object('Averages', 'Event ID range').data
wavelength_array = CRDS_file.object('Averages', 'Wavelength (nm)').data
ringdown_array = CRDS_file.object('Averages', 'Mean ring-down time (s)').data

CRDS_array = np.column_stack((timestamp_array, timestamp_to_zero, wavelength_array, ringdown_array))   ### Stacks each array column-wise
row_number = 0
#%%


"Remove CRDS points outside limits"

for row in CRDS_array:
           
    if not RDT_lower <= row[3] * 10**6 <= RDT_upper:  ### Checks the RDT lies within the specified limits 
        CRDS_array = np.delete(CRDS_array, row_number, axis = 0)  ### Deletes the row in question if not
    
    else:
        row_number += 1                                 ### Moves on to the next row if there is no deletion, otherwise, the deletion effectively moves on to the next row as the array becomes one row shorter
#%%
        
        
"Filter CRDS data"

if Filtering == True:
    start = float(len(CRDS_array))
    x = 0
    
    outliers_RDT = []
    outliers_time = []
    
    while x < 2:
        i = 1
        while i < len(CRDS_array):
            
            if CRDS_array[i - 1][3] * limit > CRDS_array[i][3] and CRDS_array[i + 1][3] * limit > CRDS_array[i][3]:
                outliers_RDT.append(CRDS_array[i][3])
                outliers_time.append(CRDS_array[i][1])
                CRDS_array = np.delete(CRDS_array, i, axis = 0)
                
            else:
                i += 1
        x += 1
    
    x = 0
    while x < 2:
        i = 1
        while i < len(CRDS_array):
            
            if CRDS_array[i - 2][3] * limit > CRDS_array[i][3] and CRDS_array[i + 2][3] * limit > CRDS_array[i][3]:
                outliers_RDT.append(CRDS_array[i][3])
                outliers_time.append(CRDS_array[i][1])
                CRDS_array = np.delete(CRDS_array, i, axis = 0)
                
            else:
                i += 1
        x += 1

    end = float(len(CRDS_array))
    
    unfiltered = plt.scatter(CRDS_array[:, 1], (CRDS_array[:, 3] * 10**6), marker = 'o', edgecolors = 'none', c = 'c')
    filtered = plt.scatter(outliers_time, [value * 10**6 for value in outliers_RDT], marker = 'o', edgecolors = 'none', c = 'r')
    plt.xlabel('time / s'); plt.ylabel(r'Ringdown time / $\mu$s')
    plt.title('Data filtering')
    plt.grid(b = True, which = 'major', axis = 'both', alpha = 0.5)
    plt.legend((unfiltered, filtered), ('Unfiltered', 'Filtered'), loc = 'upper right', scatterpoints = 1)
    
    plt.show()
    
    points_removed = int(start - end)
    percent_removed = (points_removed / start) * 100
    print 'Points removed by filtering: ' + str(points_removed)
    print 'Percentage removed: ' + str('%.2f' % percent_removed) + ' %'
#%%


"Extract FAGE data"

if Comparison == True:

    for file in os.listdir(FAGE):
        
        FAGE_file = pd.read_csv(FAGE + file, skiprows = 5, usecols = (0, 5, 11))   ### Imports data from FAGE analysis - time (0), concentration (11), OH/HO2/offline state (5)
    
    FAGE_array = np.asarray(FAGE_file)    ### Converts the dataframe to an array
    
    row_number = 0
    for row in FAGE_array:
        
        if row[1] != 2:
            FAGE_array = np.delete(FAGE_array, row_number, axis = 0)             ### Deletes the rows that don't correspond to HO2 measurements
        
        else:
            row_number += 1
    
    FAGE_array = np.delete(FAGE_array, 1, axis = 1)                              ### Deletes the now useless FAGE state column
    
    a = FAGE_array[:, 0] + time_offset
    FAGE_array = np.insert(FAGE_array, 1, a, axis = 1)
    
    row_number = 0
    for row in FAGE_array:
        
        if row[1] < 0:
            FAGE_array = np.delete(FAGE_array, row_number, axis = 0)
        
    else:
        row_number += 1

    
"Calculate - CRDS data"

alpha_array    = []
HO2_array      = []
lin_background = []

for row in CRDS_array:
    
    lin_background.append((row[1] * baseline_gradient) + baseline_intercept)   ### Uses line equation to generate a baseline

CRDS_array = np.column_stack((CRDS_array, lin_background))                               ### Adds the baseline to the array

for row in CRDS_array:
    
    ans = Absorbance(row[3], row[4], sigma, speed_of_light)                    ### Calculates alpha and HO2 for each row in the array
    alpha_array.append(ans[0])
    HO2_array.append(ans[1])

CRDS_array = np.column_stack((CRDS_array, alpha_array, HO2_array))                       ### Adds the alpha and HO2 columns to the main array

#%%
"Trim arrays to match length"

if Comparison == True:

    ### Trim start ###
    CRDS_start = float(CRDS_array[0][1])
    FAGE_start = float(FAGE_array[0][1])
    
    Boolean = CRDS_start < FAGE_start
    
    if Boolean == True:
        
        row_number = 0
        for row in CRDS_array:
            if row[1] < FAGE_start:
                CRDS_array = np.delete(CRDS_array, row_number, axis = 0) 
            else:
                row_number += 1
    
    elif Boolean == False:
        
        row_number = 0
        for row in FAGE_array:
            if row[1] < CRDS_start:
                FAGE_array = np.delete(FAGE_array, row_number, axis = 0) 
            else:
                row_number += 1    
    
    
    ### Trim end ###
    CRDS_end = float(CRDS_array[-1][1])
    FAGE_end = float(FAGE_array[-1][1])
    Boolean = CRDS_end > FAGE_end
    
    if Boolean == True:
        
        row_number = 0
        for row in CRDS_array:
            if row[1] > FAGE_end:
                CRDS_array = np.delete(CRDS_array, row_number, axis = 0) 
            else:
                row_number += 1
    
    elif Boolean == False:
        
        row_number = 0
        for row in FAGE_array:
            if row[1] > CRDS_end:
                FAGE_array = np.delete(FAGE_array, row_number, axis = 0) 
            else:
                row_number += 1
                
    ### Set starts to zero ###
    
    a = CRDS_array[:, 0] - CRDS_array[0][0]
    CRDS_array = np.delete(CRDS_array, 1, axis = 1)
    CRDS_array = np.insert(CRDS_array, 1, a, axis = 1)
    
    a = FAGE_array[:, 1] - FAGE_array[0][1]
    FAGE_array = np.delete(FAGE_array, 1, axis = 1)
    FAGE_array = np.insert(FAGE_array, 1, a, axis = 1)

    
    "Choose time range to analyse"
    
    row_number = 0

    for row in CRDS_array:
        if not time_start < row[1] < time_end:
            CRDS_array = np.delete(CRDS_array, row_number, axis = 0)  ### If not, delete the row   
        else:
            row_number += 1
    
    row_number = 0
    for row in FAGE_array:
        if not time_start < row[1] < time_end:
            FAGE_array = np.delete(FAGE_array, row_number, axis = 0)
        else:
            row_number += 1
    

    
    
    "Average - CRDS data"
    
    ans = intervallic_average(CRDS_array[:, 1 ], CRDS_array[:, 6], averaging_interval)        ### Runs through the array and applies the averaging function
    CRDS_averaged_times  = ans[0]
    CRDS_averaged_signal = ans[1]
    CRDS_averaged_dev    = ans[2]
    CRDS_points_averaged = ans[3]
    
    CRDS_averaged_array = np.column_stack((CRDS_averaged_times, CRDS_averaged_signal, CRDS_averaged_dev, CRDS_points_averaged))   ### Stacks the results into a new array
    
           
    "Average - FAGE data"
        
    ans = intervallic_average(FAGE_array[:, 1], FAGE_array[:, 2], averaging_interval)
    FAGE_averaged_times  = ans[0]
    FAGE_averaged_signal = ans[1]
    FAGE_averaged_dev    = ans[2]
    FAGE_points_averaged = ans[3]    
    
    FAGE_averaged_array = np.column_stack((FAGE_averaged_times, FAGE_averaged_signal, FAGE_averaged_dev, FAGE_points_averaged))
   
    averaged_array = np.column_stack((CRDS_averaged_array, FAGE_averaged_array))   ### Stacks both averaged arrays together for easy writing to file

    row_number = 0
    points_removed = 0
    for row in averaged_array:
        if row[3] < minimum_points or row[7] < minimum_points:
            averaged_array = np.delete(averaged_array, row_number, axis = 0)
            points_removed += 1
        else:
            row_number += 1

if Comparison == False:
    

    "Average - CRDS data"
    
    ans = intervallic_average(CRDS_array[:, 1 ], CRDS_array[:, 6], CRDS_array[:, 5], averaging_interval)        ### Runs through the array and applies the averaging function
    CRDS_averaged_times  = ans[0]
    CRDS_averaged_signal = ans[1]
    CRDS_averaged_dev    = ans[2]
    CRDS_points_averaged = ans[3]
    CRDS_absorption_avg  = ans[4]
    CRDS_absorption_dev  = ans[5]
    
    CRDS_averaged_array = np.column_stack((CRDS_averaged_times, CRDS_averaged_signal, CRDS_averaged_dev, CRDS_points_averaged, CRDS_absorption_avg, CRDS_absorption_dev))   ### Stacks the results into a new array
    
   
    
#%%   
    
"Generate save file"

date_a = date[:4]
date_b = date[5:7]   ### Rearranges the date to ddmmyyyy format
date_c = date[8:] 
date_rearranged = date_c + date_b + date_a

time_stripped = time.replace(':', '') ### Removes the ':' characters from the time string to make it compatible with file names
    
new_output_path = Output + str(date_rearranged) + '_' + str(time_stripped) + '_CRDS_analysed/'            ### New folder for data
if not os.path.exists(new_output_path):                                        ### If folder doesn't already exists, make it
    os.mkdir(new_output_path)
        
"Generate plots"
       
if CRDS_array[0][2] == CRDS_array[1][2]:
    
    plt.scatter(CRDS_array[:, 1], (CRDS_array[:, 3] * 10**6), marker = 'o', edgecolors = 'none', c = 'c')
    plt.xlabel('time / s'); plt.ylabel(r'Ringdown time / $\mu$s')
    plt.title('Ringdown times')
    plt.grid(b = True, which = 'major', axis = 'both', alpha = 0.5)
    
    if Save == True:
            plt.savefig(new_output_path + date_rearranged + '_' + time_stripped + '_RDT.pdf')


    plt.show()
#%%
if Comparison == True:
    
        CRDS_plot = plt.scatter(CRDS_array[:, 1], CRDS_array[:, 6], marker = 'o', c = 'c', edgecolors = 'none')
        FAGE_plot = plt.scatter(FAGE_array[:, 1], FAGE_array[:, 2], marker = 'o', c = 'r', edgecolors = 'none')
        plt.xlim(xmin = min(CRDS_array[:, 1]), xmax = max(CRDS_array[:, 1]))
        plt.xlabel('time / s'); plt.ylabel('[HO2] / molecule cm-3')
        plt.title('HO2 concentration')
        plt.legend((CRDS_plot, FAGE_plot), ('CRDS', 'FAGE'), loc = 'upper right', scatterpoints = 1)
        plt.grid(b = True, which = 'major', axis = 'both', alpha = 0.5)
    
        if Save == True:
            plt.savefig(new_output_path + date_rearranged + '_Concentration.pdf')
    
        plt.show()
#%%          
        
        CRDS_plot = plt.scatter(averaged_array[:, 0], averaged_array[:, 1], marker = 'o', c = 'c', edgecolors = 'none', zorder = 1)
        FAGE_plot = plt.scatter(averaged_array[:, 4], averaged_array[:, 5], marker = 'o', c = 'r', edgecolors = 'none', zorder = 1)
        plt.errorbar(averaged_array[:, 0], averaged_array[:, 1], yerr = averaged_array[:, 2], fmt = 'none', ecolor = 'c', zorder = 0)
        plt.errorbar(averaged_array[:, 4], averaged_array[:, 5], yerr = averaged_array[:, 6], fmt = 'none', ecolor = 'r', zorder = 0)
        plt.xlim(xmin = min(CRDS_array[:, 1]), xmax = max(CRDS_array[:, 1]))
        plt.xlabel('time / s'); plt.ylabel('[HO2] / molecule cm-3')
        plt.title('Data averaged over ' + str(averaging_interval) + ' second intervals')
        plt.legend((CRDS_plot, FAGE_plot), ('CRDS', 'FAGE'), loc = 'upper right', scatterpoints = 1)
        plt.grid(b = True, which = 'major', axis = 'both', alpha = 0.5)
    
        if Save == True:
            plt.savefig(new_output_path + date_rearranged + '_' + str(averaging_interval) + '_averaging.pdf')
    
        plt.show()
        
        print 'Data averaged over ' + str(averaging_interval) + ' seconds'
        print 'CRDS maximum deviation: ' + str('%.2e' % max(averaged_array[:, 2])) + ' molecule cm-3'
        print 'FAGE maximum deviation: ' + str('%.2e' % max(averaged_array[:, 6])) + ' molecule cm-3'
        
        
        comparison_plot = plt.scatter(averaged_array[:, 5], averaged_array[:, 1], marker = 'o', c = 'c', edgecolors = 'none', zorder = 1)
        plt.errorbar(averaged_array[:, 5], averaged_array[:, 1], xerr = averaged_array[:, 6], yerr = averaged_array[:, 2], fmt = 'none', ecolor = 'b', zorder = 0)
        plt.xlabel('FAGE signal / molecule cm-3'); plt.ylabel('CRDS signal / molecule cm-3')
        plt.title('CRDS vs FAGE')
        fit = np.polyfit(averaged_array[:, 5], averaged_array[:, 1], deg = 1, full = True)
        gradient = fit[0][0]
        intercept = fit[0][1]
        x1 = [np.min(averaged_array[:, 5]), np.max(averaged_array[:, 5])]
        y1 = [xx * gradient + intercept for xx in x1]
        plt.plot(x1, y1, '-r')
        plt.annotate(('Gradient: ' + (str(('%.2f' % gradient))) + '\nIntercept: ' + str(('%.2e' % intercept))), xycoords = 'figure fraction',  horizontalalignment = 'left', verticalalignment = 'top', xy = (0.15, 0.85))
        plt.grid(b = True, which = 'major', axis = 'both', alpha = 0.5)
    
        if Save == True:
            plt.savefig(new_output_path + date_rearranged + '_' + str(averaging_interval) + '_agreement_plot.pdf')
    
        plt.show()
    
#    else:        ### Plots the data for scan measurements
#        
#        scan_RDT = plt.scatter(CRDS_array[:, 2], (CRDS_array[:, 3] * 10**6), marker = 'o', edgecolors = 'none', c = 'c')    
#        plt.xlabel('Wavelength / nm');plt.ylabel('Ringdown time / us')
#        plt.title('Ringdown times')
#        plt.show()
#        
#    #    plt.scatter(CRDS_array[:, 2], CRDS_array[:, 5])
#    #    plt.xlabel('Wavelength / nm'); plt.ylabel('alpha')
#    #    plt.title('Alpha')
#    #    plt.show()
#        
#        plt.scatter(CRDS_array[:, 2], CRDS_array[:, 6], marker = 'o', edgecolors = 'none')
#        plt.xlabel('Wavelength / nm'); plt.ylabel('[HO2] / molecule cm-3')
#        plt.title('HO2 concentration')
#        plt.show()
#        
#        CRDS_array = np.delete(CRDS_array, 0, axis = 1)   ### Deletes the first column of the array - discards original time column
#        CRDS_array = np.delete(CRDS_array, 0, axis = 1)   ### Deletes the first column of the array - discards time
    

"Write output file"

if Comparison == True:

    if CRDS_array[0][2] == CRDS_array[1][2] and Save == True:   ### Saves fixed wavelength data 
    
        CRDS_array = np.delete(CRDS_array, 0, axis = 1)   ### Deletes the first column of the array - discards original time column
        
        outfile = open(new_output_path + date_rearranged + '_' + time_stripped + '_CRDS_analysis.csv', 'w')
        
        outfile.write('Python CRDS analysis,' + date + ',' + time + '\n')
        outfile.write('Baseline parameters:, Gradient:,' + str(baseline_gradient))
        outfile.write('\n, Intercept:,' + str(baseline_intercept) + '\n')
        outfile.write('Cross section:,' + str(sigma) + ', cm2 \n')
        outfile.write('FAGE time offset:,' + str(time_offset) + ',seconds\n\n')
        outfile.write('Time / s , Wavelength / nm , RDT / us , Lin_Baseline / us , alpha / cm-1 , [HO2] / molecule cm-3 \n\n')
        np.savetxt(outfile, CRDS_array, delimiter = ',')
        
        outfile.close()
        
        
        "Write averaged output file"
        
        outfile = open(new_output_path + date_rearranged + '_' + time_stripped + '_CRDS_' + str(averaging_interval) + '_second_averaged.csv', 'w')
        
        outfile.write('Python CRDS averaged,' + date + ',' + time + '\n')
        outfile.write('Averaging time:,' + str(averaging_interval) + ',seconds\n')
        outfile.write('Baseline parameters:, Gradient:,' + str(baseline_gradient))
        outfile.write('\n, Intercept:,' + str(baseline_intercept) + '\n')
        outfile.write('Cross section:,' + str(sigma) + ', cm2 \n')
        outfile.write('FAGE time offset:,' + str(time_offset) + ',seconds \n')
        outfile.write('Comparison parameters:, Gradient:, ' + str(gradient))
        outfile.write('\n, Intercept:,' + str(intercept) + '\n\n')
        
        outfile.write('CRDS Time / s , CRDS [HO2] / molecule cm-3 , CRDS [HO2] error / molecule cm-3, CRDS points averaged, FAGE Time / s , FAGE [HO2] / molecule cm-3 , FAGE [HO2] error / molecule cm-3, FAGE points averaged \n\n')
        np.savetxt(outfile, averaged_array, delimiter = ',')
        
        outfile.close()
    
    elif CRDS_array[0][2] != CRDS_array[1][2] and Save == True:   ### Saves scan data   
        
        outfile = open(new_output_path + date_rearranged + '_' + time_stripped + '_CRDS_scan_analysis.csv', 'w')
    
        outfile.write('Python CRDS analysis,' + date + ',' + time + '\n')
        outfile.write('Wavelength / nm , RDT / us , Baseline / us, alpha / cm-1 , [HO2] / molecule cm-3 \n\n')
        np.savetxt(outfile, CRDS_array, delimiter = ',')
        
        outfile.close()

elif Comparison == False:
    
    if CRDS_array[0][2] == CRDS_array[1][2] and Save == True:   ### Saves fixed wavelength data 
    
        CRDS_array = np.delete(CRDS_array, 0, axis = 1)   ### Deletes the first column of the array - discards original time column
        
        outfile = open(new_output_path + date_rearranged + '_' + time_stripped + '_CRDS_analysis.csv', 'w')
        
        outfile.write('Python CRDS analysis,' + date + ',' + time + '\n')
        outfile.write('Baseline parameters:, Gradient:,' + str(baseline_gradient))
        outfile.write('\n, Intercept:,' + str(baseline_intercept) + '\n')
        outfile.write('Cross section:,' + str(sigma) + ', cm2 \n')
        
        outfile.write('Time / s , Wavelength / nm , RDT / us , Lin_Baseline / us , alpha / cm-1 , [HO2] / molecule cm-3 \n\n')
        np.savetxt(outfile, CRDS_array, delimiter = ',')
        
        outfile.close()
        
    outfile = open(new_output_path + date_rearranged + '_' + time_stripped + '_CRDS_averaging.csv', 'w')
    
    outfile.write('Python CRDS averaging,' + date + ',' + time + '\n')
    outfile.write('Baseline parameters:, Gradient:,' + str(baseline_gradient))
    outfile.write('\n, Intercept:,' + str(baseline_intercept) + '\n')
    outfile.write('Cross section:,' + str(sigma) + ', cm2 \n')
    
    outfile.write('Time / s , Average signal , Average deviation , Points averaged, Absorption Avg, Absorption Deviation \n\n')
    np.savetxt(outfile, CRDS_averaged_array, delimiter = ',')
    
    outfile.close()
