"N2O Actinometry Data Analysis Script --- Python 2.7"
"Alex Brennan"


# To use the program, first set up the two file handles (Calrack and Output) that lead to your Calrack data file, and to where you want the output file, respectively.  These need to be sparate folders.
# Set the fraction of N2/O2 used in the main flow
# Check that the calibration data is correct ('NOx box calibrations')
# Run the program
# An excel file (.csv) will be generated in the directory specified (Output)


"File handles --- Paths must be within '' marks, replace the '\' symbols with '/' and avoid spaces"

Calrack    = 'C:/Users/cm11ab/Desktop/Working_File/N2O_Actinometry/Data/'  ### Sets file path to calrack file

Output     = 'C:/Users/cm11ab/Desktop/Working_File/N2O_Actinometry/'       ### Sets filepath to output folder

Title = '27/07/2017 Actinometry 20 SLM air'                                                            ### Names the file


"Switches"

divide_time = True


"Assign variables"

### Main Flow Composition ###
N2fract = 0.8
O2fract = 0.2
 
### NOx box calibrations                ### Data from 02/08/2017                                  ### Data from 24/11/2016              ### Data from 06/05/2016
NO_Cal_Grad  = 0.811                    ###0.6516                                                 ### 0.5691                            ### 0.4443    ### NOx box calibration gradient
NO_Cal_Int   = 0.044                    ###-0.3635                                                ### -0.1327                           ### 0.2442    ### NOx box calibration intercept
N2O_Cal_Grad = -0.0013                  ###-0.0040                                                ### -0.0033                           ### -0.0016   ### N2O quenching calibration gradient
N2O_Cal_Int  = 0.7684                   ###0.6617                                                 ### 0.5710                            ### 0.4747    ### N2O quenching calibration intercept

### Constants ###
R     = 8.31447                     ### Gas constant (J mol-1 K-1)
R     = (R * 10**6 * 760 / 101325)  ### Converts R units (cm3 Torr mol-1 K-1)
T     = 25                          ### Gas temperature (C) - will be overwritten if T is recorded
P     = 760                         ### Pressure (Torr)
Na    = 6.02214E23                  ### Avogadro's number (mol-1)
sigma = 1.43E-19                    ### Absorption cross section of N2O (cm2 molecule-1)
 
### Wand dimensions ###
l = 3.6             ### Container Photolysis length (cm)
w = 1.27            ### Container Wand width from side to side (cm)
h = 1.27            ### Container Wand height from window to top wall (cm)
V = w * h * l       ### Container Wand photolysis volume (cm3)


"Import modules"

import numpy as np
import os
import matplotlib.pyplot as plt


"Initialise lists"

LampCurrent = []        ### Initialises lamp current list (mA)
NOppblist   = []        ### Initialises [NO] list (ppb)
NOzero      = []        ### Initialises zero list (ppb)
NOcorrlist  = []        ### Initialises zero corrected list (ppb)
Flux        = []        ### Initialises flux list (photons cm-2 s-1)
x           = 0         ### Counts lines analysed


"Alanlyse data"

for file in os.listdir(Calrack):
    f = np.genfromtxt(Calrack + file, delimiter = ',', skip_header = (1))   ### Imports data as array
    
    for line in f:
        if line[0] < 0.05:                 ### Scans data for zero readings - zero current should vary below 0.05 mA
            NOzero.append(line[12])        ### Adds zero readings to zero list        
    zeroavg = np.average(NOzero)           ### Averages all zero readings 
    
    for line in f:                         ### Rescans file to gather all data
        
        LampCurrent.append(line[0])        ### Generates lamp current list
        FlowTot = line[2] + line[3]        ### Calculates total flow rate (sccm)
        N2Ofract = line[2] / FlowTot       ### Calculates N2O fraction
        T = line[8]                        ### Reads T for use in calculations (C)
        P = line[9]                        ### Reads P for use in calculations (Torr)
        
        rho = (P / (T + 273.15)) * (1 / R) * Na    ### Calculates gas density (molecules cm-3)
        N2O = rho * N2Ofract                       ### Calculates [N2O] (molecule cm-3)
        NO  = line[12]                             ### Reads [NO] (ppb)
        NOppblist.append(NO)                       ### Adds to [NO] list
        NO -= zeroavg                              ### Subtracts average zero
        
        NOcorr = NO / ((N2Ofract * 100 * N2O_Cal_Grad) + N2O_Cal_Int)
        NOcorrlist.append(NOcorr)
        NO = NOcorr * 1E-9 * rho               ### Converts from ppb to molecule cm-3
       
        O2 = rho * (1 - N2Ofract) * O2fract    ### Calculates O2 concentration i.e 20% (molecule cm-3)
        N2 = rho * (1 - N2Ofract) * N2fract    ### Calculates N2 concentration i.e 80% (molecule cm-3)
       
        k1 = 3.30E-11 * np.exp(55 / (273 + T))    ### O1D + O2  -> O3P + O2
        k2 = 2.15E-11 * np.exp(110 / (273 + T))   ### O1D + N2  -> O3P + N2
        k3 = 4.63E-11 * np.exp(20 / (273 + T))    ### O1D + N2O ->  N2 + O2
        k4 = 7.25E-11 * np.exp(20 / (273 + T))    ### O1D + N2O -> 2NO
        rateN2O = (k3 + k4) * N2O                 ### Calculates total rate of N2O loss
        
        Ft = (k1 * O2 + k2 * N2 + rateN2O) * (NO / (2 * k4 * sigma * N2O**2))  ### Calculates Flux x t (photons cm-2)
        t = (V / FlowTot) * 60 * (273.15 / (273.15 + T))                       ### Calculates irradiation time (s) using volumetric flow rate
        F = Ft / t                                                             ### Calculates flux (photons cm-2 s-1)
        absorbed = np.exp(-sigma * N2O * h)                                    ### Calculates the fraction of photons that are absorbed
        corrF = (F / (1 + absorbed) * 2)                                       ### Corrects flux for absorption of light by N2O.  The factor of 2 estimates the average flux at the centre of the wand
        
        if divide_time == True:
            Flux.append(corrF)
        elif divide_time == False:
            Flux.append(Ft)             
        
        x += 1
        

"Plot results"

plt.scatter(LampCurrent, Flux)
fit = np.polyfit(LampCurrent, Flux, 2)             ### Determines curve parameters
B1     = fit[1]
B2     = fit[0]
iCurve = fit[2]
x1 = np.array(range(0, int(max(LampCurrent))))     ### Cannot use floats, so line may not look perfect 
y1 = eval('iCurve + (B1 * x1) + (B2 * x1**2)')     ### However, the line parameters are determined properly
plt.plot(x1, y1, '-b')

#plotlist = []                               ### Not necessary with linear lamp output   
#fluxlist = []
#i        = 0
#while i < len(LampCurrent):                 ### Generates list of lamp currents below 5 mA
#    if LampCurrent[i] < 5:                  ### This is then used to do a linear plot over the low current range
#        plotlist.append(LampCurrent[i])
#        fluxlist.append(Flux[i])
#    i += 1
#
#fit = np.polyfit(plotlist, fluxlist, deg = 1, full = True)
#slope = fit[0][0]                                               ###
#intercept = fit[0][1]                                           ### Line fitting.
#x1 = [0, max(plotlist)]                                         ### 
#y1 = [slope * xx + intercept for xx in x1]                      ###
#plt.plot(x1, y1, '-r')                                          ###

fit = np.polyfit(LampCurrent, Flux, deg = 1, full = True)
slope = fit[0][0]                                               ###
intercept = fit[0][1]                                           ### Line fitting.
x1 = [0, max(LampCurrent)]                                         ### 
y1 = [slope * xx + intercept for xx in x1]                      ###
plt.plot(x1, y1, '-r')                                          ###

plt.xlabel('Lamp Current / mA')
if divide_time == True:
    plt.ylabel('Flux / photons cm-2 s-1')
elif divide_time == False:
    plt.ylabel('F x t / photons cm-2')
plt.title(Title)
plt.grid(b = True, which = 'major', axis = 'both', alpha = 0.5)
plt.annotate(('Curve: \nB1: ' + (str(('%.2e' % B1))) + '\nB2: ' + (str(('%.2e' % B2))) +'\nIntercept: ' + str(('%.2e' % iCurve))), xycoords = 'figure fraction',  horizontalalignment = 'left', verticalalignment = 'top', xy = (0.15, 0.85))
plt.annotate(('Linear: \nGradient: ' + (str(('%.2e' % slope))) + '\nIntercept: ' + str(('%.2e' % intercept))), xycoords = 'figure fraction',  horizontalalignment = 'left', verticalalignment = 'bottom', xy = (0.45, 0.15))

plt.savefig(Output + file.strip('.txt') + '_plot.pdf')
plt.show()

print 'File:', file
print 'N2O %:', '{:.2f}'.format(N2Ofract * 100)     ### Displays % N2O to 2 dp.
print 'Data points:', x
print 'Zero:', '{:.2f}'.format(zeroavg), 'ppb'      ### Displays zero readings to 2 dp.
print '\n'
print 'Linear Plot Data - Red'
print 'gLin:', '{:.2e}'.format(slope)               ### Displays gradient to 2 dp. in scientific notation
print 'iLin:', '{:.2e}'.format(intercept)           ### Displays intercept to 2 dp. in scientific notation
print '\n'
print 'Curved Plot Data - Blue'
print 'B1:', '{:.2e}'.format(B1)
print 'B2:', '{:.2e}'.format(B2)
print 'iCurve:', '{:.2e}'.format(iCurve)


"Generate output file"

output = open(Output + file.strip('.txt') + '.csv', 'w')

output.write(Title + '\n')

output.write('NO cal parameters:\n Gradient: ,' + str(N2O_Cal_Grad) + '\n Intercept: ,' + str(N2O_Cal_Int))

output.write('\n\nLamp Current / mA, Flux / photons cm-2 s-1 \n')

i = 0
while i < x:
    output.write(str(LampCurrent[i]))
    output.write(',')                       ### Adds comma delimiter
    output.write(str(Flux[i]))
    output.write('\n')                      ### Carriage return
    i += 1                                  ### Adds to iteration counter

output.write('\n Linear Plot Data')
output.write('\n Gradient:')
output.write(',')               
output.write(str(slope))                    ### Converts to string and writes gradient
output.write('\n Intercept:')
output.write(',')
output.write(str(intercept))                ### Converts to string and writes intercept

output.write('\n \n Curved Plot Data')
output.write('\n B1:') 
output.write(',')
output.write(str(B1))
output.write('\n B2:')
output.write(',')
output.write(str(B2))
output.write('\n Intercept:')
output.write(',')
output.write(str(iCurve))

output.close()

      