"NOxBOX calibration"


"Input file paths"

Data         = 'C:\Users\cm11ab\Desktop\Working_File\NOxBOX_Calibrations\Data'
Output       = 'C:\Users\cm11ab\Desktop\Working_File\NOxBOX_Calibrations'
Output_label = '466_ppb'


"Input pameters"

NO_concentration       = 466                                                   ### NO standard concentration (ppb)
NO_concentration_error = 4.66  # 1% error                                      ### NO standard concentration error (± ppb)
N2O_concentration      = 0.98

"Switches"

save_plots = True


"Flow controller corrections"

AirMFCCal       = 0.9922
AirMFCOffset    = 0.5323
AirMFCGasFactor = 1

NOMFCCal        = 0.9481
NOMFCOffset     = -0.0019
NOMFCGasFactor  = 0.995

N2OMFCCal       = 0.7301
N2OMFCOffset    = -0.0055
N2OMFCGasFactor = 1                                                            ### Set to 1 as factor is incorporated into acquisition software (0.750)  


Data = Data.replace('\\', '/')
Data = Data + '/'
Output = Output.replace('\\', '/')
Output = Output + '/'


"Import modules"

import os
import numpy as np
import matplotlib.pyplot as plt


"Define functions"

def calculate(Air, NO, N2O):                                                   ### Uses the MFC calibration factors and gas correction factors to correct the gas flows 
    actual_Air    = ((Air * AirMFCCal) + AirMFCOffset) * AirMFCGasFactor
    actual_NO     = ((NO * NOMFCCal) + NOMFCOffset) * NOMFCGasFactor
    actual_N2O    = ((N2O * N2OMFCCal) + N2OMFCOffset) * N2OMFCGasFactor * N2O_concentration
    total_flow    = actual_Air + actual_NO + actual_N2O
    calculated_NO = NO_concentration * (actual_NO / total_flow)
    return actual_Air, actual_NO, actual_N2O, calculated_NO
    
def average(Air, NO, N2O, NO_calculated, NO_measured):
    average_Air = []; error_Air = []; average_NO = []; error_NO = []; average_N2O = []; error_N2O = [] 
    average_NO_calc = []; error_NO_calc = []; average_NO_meas = []; error_NO_meas = []
    for index, list in enumerate(Air):
        average_Air.append(np.average(list));                     error_Air.append(np.std(list))
        average_NO.append(np.average(NO[index]));                 error_NO.append(np.std(NO[index]))
        average_N2O.append( np.average(N2O[index]));              error_N2O.append(np.std(N2O[index]))
        average_NO_calc.append(np.average(NO_calculated[index])); error_NO_calc.append(np.std(NO_calculated[index]))
        average_NO_meas.append(np.average(NO_measured[index]));   error_NO_meas.append(np.std(NO_measured[index]))
    return average_Air, error_Air, average_NO, error_NO, average_N2O, error_N2O, average_NO_calc, error_NO_calc, average_NO_meas, error_NO_meas


"Open output file"

output_file = open(Output + 'Outfile_' + Output_label + '.csv', 'w')
output_file.write('Main flow / sccm, error Main flow, N2O flow / sccm, error N2O flow, NO flow / sccm, error NO flow, NO calculated / ppb, error NO calculated, NO measured / ppb, error NO measured, subtracted zero / ppb \n')


"Initialise main lists"

N2O_fractions  = []
N2O_gradients  = []
N2O_intercepts = []


"Read data from files"

for file in os.listdir(Data):
    
    main_flow_list  = []                                                       ### Nested lists containing data for each set of NO measurements within each file
    NO_flow_list    = []
    N2O_flow_list   = []
    NO_actual_list  = []
    NO_reading_list = []
    
    main_flow  = []                                                            ### Lists of all data in the current file
    NO_flow    = []
    N2O_flow   = []
    NO_reading = []
    
    f = np.genfromtxt(Data + file, delimiter = ',', skip_header = 1)
    
    for line in f:
        main_flow.append(line[4])
        NO_flow.append(line[3])
        N2O_flow.append(line[2])
        NO_reading.append(line[5])

    N2O_flow = [x if x > 5 else 0 for x in N2O_flow]                           ### Replaces the offset values with zero (They should all be less than 5, may need tweeking)
    NO_flow  = [x if x > 5 else 0 for x in NO_flow]                            ### Same again
    

    "Analyse data"
    
    main = []; NO = []; N2O = []; NO_measured = []; NO_actual  = []            ### Lists for each set NO flow rate.  Once the flow rate is changed, these lists are appended to the main lists and then reset, ready for the next NO flow rate.
    
    for i, v in enumerate(NO_flow):                                            ### Works through the file lists, separating them into nested lists for each NO flow rate
        try:
            if NO_flow[i + 1] * 0.90 <= v <= NO_flow[i + 1] * 1.10:
                main.append(main_flow[i]); NO.append(v); N2O.append(N2O_flow[i]); NO_measured.append(NO_reading[i])
                NO_actual.append(calculate(main_flow[i], v, N2O_flow[i])[3])
            else:
                main_flow_list.append(main); NO_flow_list.append(NO); N2O_flow_list.append(N2O); NO_actual_list.append(NO_actual); NO_reading_list.append(NO_measured)
                main = []; NO = []; N2O = []; NO_measured = []; NO_actual  = []
        except:                                                                ### Allows for the last line in the file having no i + 1
            main.append(main_flow[i]); NO.append(v); N2O.append(N2O_flow[i]); NO_measured.append(NO_reading[i])
            NO_actual.append(calculate(main_flow[i], v, N2O_flow[i])[3])
            main_flow_list.append(main); NO_flow_list.append(NO); N2O_flow_list.append(N2O); NO_actual_list.append(NO_actual); NO_reading_list.append(NO_measured)
            
    ans = average(main_flow_list, NO_flow_list, N2O_flow_list, NO_actual_list, NO_reading_list)
    
    minus_zero = list(ans[8] - ans[8][ans[2].index(0)])                        ### Finds the zero NO set point in the list (ans 2) and subtracts the corresponding NO reading (ans 8) from all readings.  List converts the resulting array back into a list
    
    i = 0
    N2O_fraction_list = []                                                     ### Calculates the average N2O percentage for each file based upon flow rates
    while i < len(ans[0]):    
        N2O_fraction_list.append(ans[4][i] / (ans[0][i] + ans[2][i] + ans[4][i]))
        i += 1
        
    N2O_fraction = np.average(N2O_fraction_list) * 100

    plt.title('NOx Box Calibration ' + str('%.2f' % N2O_fraction) + ' % N2O')
    plt.xlabel('NO calculated / ppb')
    plt.ylabel('NO measured / ppb')
    plt.scatter(ans[6], minus_zero)
   
    fit = np.polyfit(ans[6], minus_zero, deg = 1, full = True)
    gradient  = fit[0][0]
    intercept = fit[0][1]
    
    x1 = [0, max(ans[6])]                         
    y1 = [gradient * xx + intercept for xx in x1]              
    plt.plot(x1, y1, '-r') 
    plt.grid(b = True, which = 'major', axis = 'both', alpha = 0.5)
    plt.annotate(('Gradient: ' + (str(('%.2e' % gradient))) + '\nIntercept: ' + str(('%.2e' % intercept))), xycoords = 'figure fraction',  horizontalalignment = 'left', verticalalignment = 'top', xy = (0.15, 0.85))
    plt.savefig(Output + '{0:.0f}'.format(N2O_fraction) + 'pct.pdf')    
    plt.show()
       
    N2O_fractions.append(N2O_fraction)
    N2O_gradients.append(gradient)
    N2O_intercepts.append(intercept)    
    
    print file
    print '{0:.0f}'.format(N2O_fraction), '% N2O Gradient: ', gradient
    print '{0:.0f}'.format(N2O_fraction), '% N2O Intercept: ', intercept    
    
    
    "Write to output file"
    
    output_file.write(file + ', % N2O,' + str('{0:.0f}'.format(N2O_fraction)) + '\n')    
    
    for i, v in enumerate(ans[0]):
        output_file.write(str(ans[0][i]) + ',' + str(ans[1][i]) + ',' + str(ans[4][i]) + ',' + str(ans[5][i]) + ',' + str(ans[2][i]) + ',' + str(ans[3][i]) + ',' + str(ans[6][i]) + ',' + str(ans[7][i]) + ',' + str(ans[8][i]) + ',' + str(ans[9][i]) + ',' + str(minus_zero[i]) + '\n')
    
    output_file.write('\n')
    output_file.write('Gradient:,' + str(gradient) + '\n' + 'Intercept:,' + str(intercept) + '\n \n')
    
plt.title('NOx Box N2O response'); plt.xlabel('N2O fraction / %'); plt.ylabel('C factor')
    
plt.scatter(N2O_fractions, N2O_gradients)
fit = np.polyfit(N2O_fractions, N2O_gradients, deg = 1, full = True)
gradient = fit[0][0]
intercept = fit[0][1]

x1 = [0, max(N2O_fractions)]                         
y1 = [gradient * xx + intercept for xx in x1]              
plt.plot(x1, y1, '-r')

plt.grid(b = True, which = 'major', axis = 'both', alpha = 0.5)
plt.annotate(('Gradient: ' + (str(('%.2e' % gradient))) + '\nIntercept: ' + str(('%.2e' % intercept))), xycoords = 'figure fraction',  horizontalalignment = 'left', verticalalignment = 'top', xy = (0.55, 0.85))
plt.savefig(Output + 'N2O response' + '.pdf')    

plt.show()

output_file.write('N2O Response \n')
output_file.write('N2O fraction, Calibration factor, Calibration intercept \n')
for i,v in enumerate(N2O_fractions):
    output_file.write(str(v) + ',' + str(N2O_gradients[i]) + ',' + str(N2O_intercepts[i]) + '\n')
output_file.write('\nResponse gradient:,' + str(gradient) + '\nResponse intercept:,' + str(intercept))

output_file.close()

print 'N2O response gradient: ', gradient
print 'N2O response intercept: ', intercept